package review.ihm.international;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Locale;
import java.util.MissingResourceException;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;

/**
 * classe illustrant l'usage des Locales pour la mise en forme des valeurs
 * numériques et des valeurs monaitaires. 
 * 
 * Ce frame contient une ComboBox comportant une liste de pairs "langue-pays" 
 * et deux labels avec une valeur numérique fixe. Chaque fois qu'on choisi un
 * pair "langue-pays" dans la ComboBox, une locale correspondant à ce pair
 * est créé et les labels sont mis en format selon cette locale.  
 * 
 * @author kirsch
 */
public class LocaleDemoFrame extends JFrame implements ActionListener {

    public static final double DEFAULT_VALUE = 15500.00;
    private double value;
    private DefaultComboBoxModel<LangCountryPeer> comboModel;
    private JLabel labelLocales;
    private JLabel labelValues;
    private JLabel labelNumber;
    private JLabel labelCurrency;
    private JComboBox comboLocale;

    public LocaleDemoFrame() {
        super();
        this.value = DEFAULT_VALUE;
        this.buildLocaleList(Locale.getAvailableLocales());
        this.initComponents();

    }

    public LocaleDemoFrame(double value) {
        super();
        this.value = value;
        this.buildLocaleList(Locale.getAvailableLocales());
        this.initComponents();
    }

    public LocaleDemoFrame(double value, Locale locales[]) {
        super();
        this.value = value;
        this.buildLocaleList(locales);
        this.initComponents();
    }

    public double getValue() {
        return this.value;
    }

    public void setValue(double value) {
        this.value = value;

        //mise à jour de l'interface
        this.formatLabelNumber(null);
        this.formatLabelCurrency(null);
    }

    public Locale[] getLocaleList() {
        Locale locales[] = new Locale[this.comboModel.getSize()];
        for (int i = 0; i < locales.length; i++) {
            LangCountryPeer peer = this.comboModel.getElementAt(i);
            locales[i] = new Locale(peer.getLang(), peer.getCountry());
        }
        return locales;
    }

    public void setLocaleList(Locale[] locales) {
        this.buildLocaleList(locales);
    }
    
    public Locale getSelectedLocale() {
        Locale loc = null;
        
        //si la combobox existe et si elle a qqch de selectionnée
        if (this.comboLocale != null && this.comboLocale.getSelectedIndex()>=0) {
            LangCountryPeer peer = (LangCountryPeer)this.comboLocale.getSelectedItem();
            loc = new Locale(peer.getLang(), peer.getCountry());
        }
        
        return loc;
    }

    public String getLocalesLabel() {
        return this.labelLocales.getText();
    }

    public void setLocaleLabel(String label) {
        this.labelLocales.setText(label);
    }

    public String getValueLabel() {
        return this.labelValues.getText();
    }

    public void setValueLabel(String label) {
        this.labelValues.setText(label);
    }
    
    private void buildLocaleList(Locale locales[]) {
        //si mise à jour de la liste ou nouveau combo
        if (this.comboModel != null) {
            //avant d'ajouter les nouveaux éléments, on supprime les anciens
            this.comboModel.removeAllElements();
        } else {
            //nouvelle liste 
            this.comboModel = new DefaultComboBoxModel<>();
        }

        //d'abord on va ordonner l'array locales
        Arrays.sort(locales, new Comparator<Locale> () {
            public int compare(Locale o1, Locale o2) {
                return o1.getLanguage().compareTo(o2.getLanguage());
            }
         });
        
        //ensuite, on le met dans le modèle de la combo
        for (Locale loc : locales) {
            String lang = loc.getLanguage();
            String country = loc.getCountry();

            //locale contient tte les infos qui nous intéressent, on la garde 
            if (!lang.isEmpty() && !country.isEmpty()) {
                this.comboModel.addElement(new LangCountryPeer(lang, country));
            }

        }
    }

    private void initComponents() {

        this.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        //premier label, celui pour la combo de locales
        c.gridx = 0;
        c.gridy = 0;
        this.labelLocales = new JLabel("Locales");
        //on definit une police légèrement plus grande en gras
        this.labelLocales.setFont(new Font (this.labelLocales.getFont().getFontName(),
                                            Font.BOLD, this.labelLocales.getFont().getSize()+1) );
        this.add(this.labelLocales, c);
        
        
        //second label, celui pour indiquer la colonne de valeurs
        c.gridx = 1;
        c.gridy = 0;
        this.labelValues = new JLabel("Valeurs");
        //on definit une police légèrement plus grande en gras
        this.labelValues.setFont(new Font (this.labelValues.getFont().getFontName(),
                                            Font.BOLD, this.labelValues.getFont().getSize()+1) );
        this.add(this.labelValues, c);

        //troisième label, celui qui contient la valeur numerique formatée
        c.gridx = 1;
        c.gridy = 1;
        this.labelNumber = new JLabel();
        this.formatLabelNumber(null);
        this.add(this.labelNumber, c);

        //quatrième label, celui qui contient la valeur formatée monnaie 
        c.gridx = 1;
        c.gridy = 2;
        this.labelCurrency = new JLabel();
        this.formatLabelCurrency(null);
        this.add(this.labelCurrency, c);

        //enfin le comboBox
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 1;
        c.gridheight = 2;
        this.comboLocale = new JComboBox(this.comboModel);
        this.add(this.comboLocale, c);

        //puis, les événements 
        this.comboLocale.addActionListener(this);

        this.pack();

        this.setSize((int) (this.getWidth() + (this.getWidth() * 0.5)),
                (int) (this.getHeight() + (this.getHeight() * 0.15)));
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Locale loc = this.getSelectedLocale(); 

        System.out.println(loc);

        this.formatLabelNumber(loc);
        this.formatLabelCurrency(loc);
    }

    public String formatNumber(double nb, Locale loc) {
        NumberFormat fmt = NumberFormat.getNumberInstance(loc);
        String valfmt = fmt.format(nb);
        return (valfmt);
    }

    public String formatCurrency(double nb, Locale loc) {
        NumberFormat fmt = NumberFormat.getCurrencyInstance(loc);
        String valfmt = fmt.format(nb);
        return (valfmt);
    }

    /*
     * formatation du label contenant la valeur numérique selon un locale donné
     */
    private void formatLabelNumber(Locale loc) {
        if (loc != null) {
            String valfmt = this.formatNumber(value, loc);
            this.labelNumber.setText(valfmt);
            System.out.println(valfmt);
        } else {
            this.labelNumber.setText(Double.toString(this.value));
        }
    }

    private void formatLabelCurrency(Locale loc) {
        if (loc != null) {
            String valfmt = this.formatCurrency(value, loc);
            this.labelCurrency.setText(valfmt);

            System.out.println(valfmt);
        } else {
            this.labelCurrency.setText(Double.toString(this.value));
        }
    }

    public class LangCountryPeer implements Comparable<LangCountryPeer> {

        private String lang;
        private String country;

        public String getLang() {
            return this.lang;
        }

        public void setLang(String lang) {
            this.lang = lang;
        }

        public String getCountry() {
            return this.country;
        }

        public void setCountry(String country) {
            this.country = country;
        }

        public LangCountryPeer(String lang, String country) {
            this.lang = lang;
            this.country = country;
        }

        public String toString() {
            return (this.lang + "-" + this.getCountry());
        }

        @Override
        public int compareTo(LangCountryPeer o) {
            return this.lang.compareTo(o.getLang());
        }
    }
}
